# encoding: utf-8

# File:	clients/sms_auto.ycp
# Package:	Configuration of sms
# Summary:	Client for autoinstallation
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: sms_auto.ycp,v 1.1.2.2 2005/04/21 20:30:33 mclark Exp $
#
# This is a client for autoinstallation. It takes its arguments,
# goes through the configuration and return the setting.
# Does not do any changes to the configuration.

# @param function to execute
# @param map/list of sms settings
# @return [Hash] edited settings, Summary or boolean on success depending on called function
# @example map mm = $[ "FAIL_DELAY" : "77" ];
# @example map ret = WFM::CallFunction ("sms_auto", [ "Summary", mm ]);
module Yast
  class SmsAutoClient < Client
    def main
      Yast.import "UI"

      textdomain "novell-sms"

      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("Sms auto started")

      Yast.import "Sms"
      Yast.include self, "novell-sms/wizards.rb"

      @ret = nil
      @func = ""
      @param = {}

      # Check arguments
      if Ops.greater_than(Builtins.size(WFM.Args), 0) &&
          Ops.is_string?(WFM.Args(0))
        @func = Convert.to_string(WFM.Args(0))
        if Ops.greater_than(Builtins.size(WFM.Args), 1) &&
            Ops.is_map?(WFM.Args(1))
          @param = Convert.to_map(WFM.Args(1))
        end
      end
      Builtins.y2debug("func=%1", @func)
      Builtins.y2debug("param=%1", @param)

      # Create a summary
      if @func == "Summary"
        @ret = Ops.get_string(Sms.Summary, 0, "")
      # Reset configuration
      elsif @func == "Reset"
        Sms.Import({})
        @ret = {}
      # Change configuration (run AutoSequence)
      elsif @func == "Change"
        #    import "OesLdap"; // FIXME: this needed anymore
        #    OesLdap::autoyast_tool = true; // FIXME: this needed anymore
        @ret = SmsAutoSequence()
      # Import configuration
      elsif @func == "Import"
        @ret = Sms.Import(@param)
      # Return actual state
      elsif @func == "Export"
        Sms.Read if Sms.Settings == nil || Sms.Settings == {}
        Sms.auto_clone = true
        @ret = Sms.Export
      # Return needed packages
      elsif @func == "Packages"
        @ret = Sms.AutoPackages
      # Read current state
      elsif @func == "Read"
        Yast.import "Progress"
        @progress_orig = Progress.set(false)
        @ret = Sms.Read
        Progress.set(@progress_orig)
      # Write givven settings
      elsif @func == "Write"
        Yast.import "Progress"
        @progress_orig = Progress.set(false)
        Sms.write_only = true
        @ret = Sms.Write
        Progress.set(@progress_orig)
      # Return if configuration  was changed
      # return boolean
      elsif @func == "GetModified"
        Sms.globals_configured = true
        @ret = Sms.globals_configured
      # Set modified flag
      # return boolean
      elsif @func == "SetModified"
        Sms.globals_configured = true
        @ret = true
      else
        Builtins.y2error("Unknown function: %1", @func)
        @ret = false
      end

      Builtins.y2debug("ret=%1", @ret)
      Builtins.y2milestone("Sms auto finished")
      Builtins.y2milestone("----------------------------------------")

      deep_copy(@ret) 

      # EOF
    end
  end
end

Yast::SmsAutoClient.new.main
