# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	clients/sms_proposal.ycp
# Package:	Configuration of sms
# Summary:	Proposal function dispatcher.
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: sms_proposal.ycp,v 1.1.2.6 2005/02/10 21:20:33 sbunch Exp $
#
# Proposal function dispatcher for sms configuration.
# See source/installation/proposal/proposal-API.txt
module Yast
  class SmsProposalClient < Client
    def main

      textdomain "novell-sms"

      Yast.import "NovellAddonInstall"
      Yast.import "NovellLDAP"
      Yast.import "NovellUtils"
      Yast.import "NovellPatterns"
      Yast.import "Progress"
      Yast.import "Sms"


      # The main ()
      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("Sms proposal started")

      @func = Convert.to_string(WFM.Args(0))
      @param = Convert.to_map(WFM.Args(1))
      @ret = {}

      # create a textual proposal
      if @func == "MakeProposal"
        @proposal = ""
        @warning = ""
        @warning_level = nil
        @force_reset = Ops.get_boolean(@param, "force_reset", false)

        if NovellUtils.EdirAskUserFlag("sms") # If edir changed, then we need to recompute the defaults.
          @force_reset = true
          Builtins.y2milestone(
            "sms.MakeProposal:force_reset set to true due to ask user flag."
          )
        end

        if @force_reset || !Sms.proposal_valid
          Sms.proposal_valid = true
          @progress_orig = Progress.set(false)
          Sms.Read
          Progress.set(@progress_orig)
        end
        @sum = Sms.Summary
        @proposal = Ops.get_string(@sum, 0, "")
        @ret = NovellAddonInstall.ConstructProposal("sms", @proposal)
      # validate the current values
      elsif @func == "ValidateValues"
        @ret = { "errors" => false }

        if  !Sms.upgradeModeOES11SP2 &&
            !Sms.upgradeModeOES2015 &&
            !Sms.upgradeModeOES2015SP1 &&
            !Sms.upgradeModeOES11SP3 &&
            !Sms.upgradeModeOES2018 &&
            !Sms.upgradeModeOES2018SP1 &&
            !Sms.upgradeModeOES2018SP2 
          #if the user needs to be warned, warn them and block on page
          if Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_ADMIN_PASSWORD",
              ""
            ) == "" ||
              Ops.get_string(Sms.Settings, "CONFIG_SMS_LDAP_SERVER", "") == ""
            Ops.set(@ret, "errors", true)
          end
        end
      # run the module
      elsif @func == "AskUser"
        Builtins.sleep(750)
        @chosen_id = Ops.get(@param, "chosen_id")
        Builtins.y2milestone("Sms proposal changing id %1", @chosen_id)

        # Configure now
        if @chosen_id == "sms--configure_now"
          Ops.set(NovellAddonInstall.products_to_configure, "sms", true)
          @ret = { "workflow_sequence" => :next }
        elsif @chosen_id == "sms--configure_later"
          Ops.set(NovellAddonInstall.products_to_configure, "sms", false)
          @ret = { "workflow_sequence" => :next }
        else
          @stored = Sms.Export
          @seq = Convert.to_symbol(WFM.CallFunction("sms", [path(".propose")]))
          Sms.Import(@stored) if @seq != :next
          Builtins.y2debug("stored=%1", @stored)
          Builtins.y2debug("seq=%1", @seq)
          @ret = { "workflow_sequence" => @seq }
        end
      # create titles
      elsif @func == "Description"
        # Determine if we should be in the proposal screen or not.
        @force_proposal = Ops.get_boolean(@param, "force_proposal", false)
        @isSelected = NovellPatterns.IsPatternInstalled("novell-sms") # FIXME: is this needed anymore?
        @isSelected = @isSelected && NovellAddonInstall.IsProductSelected("sms")
        Builtins.y2milestone("Proposal:IsSelected:novell-sms: %1", @isSelected)
        if @isSelected || @force_proposal
          @ret = {
            # Rich text title for Sms in proposals
            "rich_text_title" => _(
              "Storage Management Services (SMS)"
            ),
            # Menu title for Sms in proposals
            "menu_title"      => _(
              "&Storage Management Services (SMS)"
            ),
            "id"              => "sms"
          }
        else
          @ret = {}
        end
      # write the proposal
      elsif @func == "Write"
        Sms.Write
      else
        Builtins.y2error("unknown function: %1", @func)
      end

      # Finish
      Builtins.y2debug("ret=%1", @ret)
      Builtins.y2milestone("Sms proposal finished")
      Builtins.y2milestone("----------------------------------------")
      deep_copy(@ret) 

      # EOF
    end
  end
end

Yast::SmsProposalClient.new.main
