# encoding: utf-8

# File:	include/novell-sms/complex.ycp
# Package:	Configuration of sms
# Summary:	Dialogs definitions
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: complex.ycp,v 1.1.2.1 2004/11/23 16:19:00 sbunch Exp $
module Yast
  module NovellSmsComplexInclude
    def initialize_novell_sms_complex(include_target)
      Yast.import "UI"

      textdomain "novell-sms"

      Yast.import "Label"
      Yast.import "Popup"
      Yast.import "Wizard"
      Yast.import "UIHelper"
      Yast.import "Sms"

      Yast.include include_target, "novell-sms/helps.rb"
    end

    # Return a modification status
    # @return true if data was modified
    def Modified
      Sms.Modified
    end

    def ReallyAbort
      !Sms.Modified || Popup.ReallyAbort(true)
    end

    def PollAbort
      UI.PollInput == :abort
    end

    # Read settings dialog
    # @return `abort if aborted and `next otherwise
    def ReadDialog
      Wizard.RestoreHelp(Ops.get_string(@HELPS, "read", ""))
      # Sms::AbortFunction = PollAbort;
      ret = Sms.Read
      ret ? :next : :abort
    end

    # Write settings dialog
    # @return `abort if aborted and `next otherwise
    def WriteDialog
      Wizard.RestoreHelp(Ops.get_string(@HELPS, "write", ""))
      # Sms::AbortFunction = PollAbort;
      ret = Sms.Write
      ret ? :next : :abort
    end

    # Summary dialog
    # @return dialog result
    def SummaryDialog
      # Sms summary dialog caption
      caption = _("Sms Configuration")

      # FIXME
      summary = Sms.Summary
      unconfigured = Ops.get_list(summary, 1, [])
      configured = Ops.get_string(summary, 0, "")

      # Frame label
      #    term contents = Wizard_hw::DetectedContent(_("Sms to Configure"),
      #	    unconfigured, false, configured);
      contents = nil

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "summary", ""),
        Label.BackButton,
        Label.FinishButton
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel || ret == :back
          if ReallyAbort()
            break
          else
            next
          end
        # overview dialog
        elsif ret == :edit_button
          ret = :overview
          break
        # configure the selected device
        elsif ret == :configure_button
          # TODO FIXME: check for change of the configuration
          selected = UI.QueryWidget(Id(:detected_selbox), :CurrentItem)
          if selected == :other
            ret = :other
          else
            ret = :configure
          end
          break
        elsif ret == :next
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end

      deep_copy(ret)
    end

    # Overview dialog
    # @return dialog result
    def OverviewDialog
      # Sms overview dialog caption
      caption = _("Sms Overview")

      overview = Sms.Overview

      # FIXME table header
      contents = UIHelper.EditTable(
        # Table header
        Header(_("Number"), _("Sms")),
        overview,
        nil,
        nil,
        nil,
        nil
      )

      contents = UIHelper.SpacingAround(contents, 1.5, 1.5, 1.0, 1.0)

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "overview", ""),
        Label.BackButton,
        Label.FinishButton
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # add
        elsif ret == :add_button
          # FIXME
          ret = :add
          break
        # edit
        elsif ret == :edit_button
          # FIXME
          ret = :edit
          break
        # delete
        elsif ret == :delete_button
          # FIXME
          next
        elsif ret == :next || ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end

      deep_copy(ret)
    end
  end
end
