# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	include/novell-sms/dialogs.ycp
# Package:	Configuration of sms
# Summary:	Dialogs definitions
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: dialogs.ycp,v 1.1.2.3 2005/01/18 23:22:43 sbunch Exp $
module Yast
  module NovellSmsDialogsInclude
    def initialize_novell_sms_dialogs(include_target)
      Yast.import "UI"

      textdomain "novell-sms"

      Yast.import "NovellLDAP"
      Yast.import "NovellLDAPUI"
      Yast.import "NovellUtils"
      Yast.import "Sms"
      Yast.import "Wizard"

      Yast.include include_target, "novell-sms/helps.rb"
      Yast.include include_target, "novell-sms/complex.rb"
    end

    def PromptIfNeededForEdirAdminPassword
      useXad = false

      if Sms.xadMode
        xad_tree_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          useXad = true
        end # if this value is set at all, then it is the eDir admin (in spite of xad name)
      end
      Builtins.y2milestone(
        "Sms.PromptIfNeededForEdirAdminPassword:use xad:%1",
        useXad
      )

      if useXad
        if Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.XadTreeAdminPasswordDialog
        end
      else
        if Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.AdminPasswordDialog
        end
      end
      Builtins.y2milestone(
        "Sms.PromptIfNeededForEdirAdminPassword:useXad %1",
        useXad
      )
      useXad
    end


    # SMS LDAP Dialog
    #
    # @return dialog result
    def LDAPDialog
      if  Sms.upgradeModeOES11SP2 ||
          Sms.upgradeModeOES2015 ||
          Sms.upgradeModeOES2015SP1 ||
          Sms.upgradeModeOES11SP3 || 
          Sms.upgradeModeOES2018 ||
          Sms.upgradeModeOES2018SP1 ||
          Sms.upgradeModeOES2018SP2 
        # skip gui on last two oes upgrade versions
        Popup.Message(NovellUtils.noConfigMessage)
        Builtins.y2milestone(
          "Sms.LDAPDialog:skipping GUI since we are in upgrade mode."
        )
        return :skip
      end

      # SMS LDAP dialog caption
      caption = _("SMS Configuration")

      ldap_server = Ops.get_string(Sms.Settings, "CONFIG_SMS_LDAP_SERVER", "")
      if ldap_server == nil || !NovellLDAP.AddressExists(ldap_server, nil)
        ldap_server = ""
      end

      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )

      server_list_widget = NovellLDAPUI.ServerListWidget(
        server_list,
        _("Directory Server Address"),
        ldap_server,
        true,
        :combo
      )
      ldap_widget = Ops.get_term(server_list_widget, "term") { VBox(:Empty) }

      contents = HVSquash(VBox(ldap_widget))


      helps = Ops.get_string(server_list_widget, "helps", "")
      Wizard.SetContentsButtons(
        caption,
        contents,
        helps,
        Label.BackButton,
        Label.NextButton
      )

      UI.SetFocus(Id(:next))

      PromptIfNeededForEdirAdminPassword()

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          ldap_server = Convert.to_string(
            UI.QueryWidget(Id(:server_select), :Value)
          )
          Ops.set(Sms.Settings, "CONFIG_SMS_LDAP_SERVER", ldap_server)

          break
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end

      deep_copy(ret)
    end
  end
end
