# encoding: utf-8

# File:	include/novell-sms/wizards.ycp
# Package:	Configuration of sms
# Summary:	Wizards definitions
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: wizards.ycp,v 1.1.2.2 2004/12/03 00:04:44 sbunch Exp $
module Yast
  module NovellSmsWizardsInclude
    def initialize_novell_sms_wizards(include_target)
      Yast.import "UI"

      textdomain "novell-sms"

      Yast.import "Sequencer"
      Yast.import "Wizard"

      Yast.include include_target, "novell-sms/complex.rb"
      Yast.include include_target, "novell-sms/dialogs.rb"
    end

    # Main workflow of the sms configuration
    # @return sequence result
    def MainSequence
      aliases = { "LDAPDialog" => lambda { LDAPDialog() } }

      sequence = {
        "ws_start"   => "LDAPDialog",
        "LDAPDialog" => { :abort => :abort, :next => :next, :skip => :next }
      }

      ret = Sequencer.Run(aliases, sequence)

      deep_copy(ret)
    end

    # Whole configuration of sms
    # @return sequence result
    def SmsSequence
      aliases = {
        "read"  => [lambda { ReadDialog() }, true],
        "main"  => lambda { MainSequence() },
        "write" => [lambda { WriteDialog() }, true]
      }

      sequence = {
        "ws_start" => "read",
        "read"     => { :abort => :abort, :next => "main" },
        "main"     => { :abort => :abort, :next => "write" },
        "write"    => { :abort => :abort, :next => :next }
      }

      Wizard.CreateDialog
      Wizard.SetDesktopIcon("sms")

      ret = Sequencer.Run(aliases, sequence)

      UI.CloseDialog
      deep_copy(ret)
    end

    # Whole configuration of sms but without reading and writing.
    # For use with autoinstallation.
    # @return sequence result
    def SmsAutoSequence
      # Initialization dialog caption
      caption = _("Sms Configuration")
      # Initialization dialog contents
      contents = Label(_("Initializing..."))

      Wizard.CreateDialog
      Wizard.SetDesktopIcon("sms")
      Wizard.SetContentsButtons(
        caption,
        contents,
        "",
        Label.BackButton,
        Label.NextButton
      )

      ret = MainSequence()

      UI.CloseDialog
      deep_copy(ret)
    end
  end
end
