# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	modules/Sms.ycp
# Package:	Configuration of sms
# Summary:	Sms settings, input and output functions
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: Sms.ycp,v 1.1.2.26 2005/09/29 21:48:18 mclark Exp $
#
# Representation of the configuration of sms.
# Input and output routines.
require "yast"

module Yast
  class SmsClass < Module
    def main
      textdomain "novell-sms"

      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "NovellAddonInstall"
      Yast.import "NovellLDAP"
      Yast.import "NovellPatterns"
      Yast.import "NovellUtils"
      Yast.import "Edirectory"
      Yast.import "EdirUtils"
      Yast.import "Popup"
      Yast.import "Progress"
      Yast.import "Service"
      Yast.import "Summary"
      Yast.import "Report"
      Yast.import "Wizard"

      @globals_configured = false

      # Settings Map
      @Settings = {}
      @oldSettings = {}
      @auto_clone = false
      @xad_packages_installed = false
      @xadMode = false

      @xadTestMode = false

      # Data was modified?
      @modified = false


      @proposal_valid = false

      # Are we in upgrade mode?
      @upgradeModeOES11SP2 = false # upgrading from OES 11 SP2
      @upgradeModeOES11SP3 = false # upgrading from OES 11 SP3
      @upgradeModeOES2015 = false # upgrading from OES 2015
      @upgradeModeOES2015SP1 = false # upgrading from OES 2015 SP1
      @upgradeModeOES2018 = false # upgrading from OES 2018
      @upgradeModeOES2018SP1 = false # upgrading from OES 2018 SP1
      @upgradeModeOES2018SP2 = false # upgrading from OES 2018 SP2

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end


    # Read all sms settings
    # @return true on success
    def Read
      # Sms read dialog caption
      caption = _("Initializing SMS Configuration")

      # TODO FIXME Set the right number of stages
      steps = 2

      sl = 500
      Builtins.sleep(sl)

      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/3
          _("Read the previous settings"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/3
          _("Read the previous settings..."),
          _("Checking dependencies..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read the previous settings
      return false if Abort()
      Progress.NextStage

      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "sms:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end

      Builtins.y2milestone("sms:xadMode:%1", @xadMode)

      # Test for the upgrade scenario
      upgradeInfo = {}
      # See if we are doing an upgrade from OES 2018 SP2
      @upgradeModeOES2018SP2 = NovellAddonInstall.IsProductUpgradeOES2018SP2(
        "sms"
      )
      if @upgradeModeOES2018SP2 == false
        # See if we are doing an upgrade from OES 2018 SP1
        @upgradeModeOES2018SP1 = NovellAddonInstall.IsProductUpgradeOES2018SP1(
          "sms"
        )
        if @upgradeModeOES2018SP1 == false
          # See if we are doing an upgrade from OES 2018
          @upgradeModeOES2018 = NovellAddonInstall.IsProductUpgradeOES2018(
            "sms"
          )
          if @upgradeModeOES2018 == false
            # See if we are doing an upgrade from OES 2015 SP1
            @upgradeModeOES2015SP1 = NovellAddonInstall.IsProductUpgradeOES2015SP1(
              "sms"
            )
            if @upgradeModeOES2015SP1 == false
              # See if we are doing an upgrade from OES 11 SP3
              @upgradeModeOES11SP3 = NovellAddonInstall.IsProductUpgradeOES11SP3("sms")
              if @upgradeModeOES11SP3 == false
                # See if we are doing an upgrade from OES 2015
                @upgradeModeOES2015 = NovellAddonInstall.IsProductUpgradeOES2015("sms")
                if @upgradeModeOES2015 == false
                  # See if we are doing an upgrade from OES 11 SP2
                  @upgradeModeOES11SP2 = NovellAddonInstall.IsProductUpgradeOES11SP2(
                    "sms"
                  )
                  if @upgradeModeOES11SP2
                    upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES11SP2(
                      "sms"
                    )
                    @oldSettings = NovellAddonInstall.GetSysconfigMap(
                      Ops.get_string(upgradeInfo, "config_file", "")
                    )
                  end #upgrading from OES 2015
                else
                  upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2015("sms")
                  @oldSettings = NovellAddonInstall.GetSysconfigMap(
                    Ops.get_string(upgradeInfo, "config_file", "")
                  )
                end #upgrading from OES 11 SP3
              else
                upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES11SP3("sms")
                @oldSettings = NovellAddonInstall.GetSysconfigMap(
                  Ops.get_string(upgradeInfo, "config_file", "")
                )
              end
            else
              upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2015SP1(
                "sms"
              )
              @oldSettings = NovellAddonInstall.GetSysconfigMap(
                Ops.get_string(upgradeInfo, "config_file", "")
              )
            end
          else
            upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018(
              "sms"
            )
            @oldSettings = NovellAddonInstall.GetSysconfigMap(
              Ops.get_string(upgradeInfo, "config_file", "")
            )
          end
        else
          upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018SP1(
            "sms"
          )
          @oldSettings = NovellAddonInstall.GetSysconfigMap(
            Ops.get_string(upgradeInfo, "config_file", "")
          )
        end
      else
        upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018SP2(
          "sms"
        )
        @oldSettings = NovellAddonInstall.GetSysconfigMap(
          Ops.get_string(upgradeInfo, "config_file", "")
        )
      end
  
      Builtins.y2milestone(
        "sms:
        upgradeModeOES11SP2 = #{@upgradeModeOES11SP2},
        upgradeModeOES2015 = #{@upgradeModeOES2015},
        upgradeModeOES2015SP1 = #{@upgradeModeOES2015SP1},
        upgradeModeOES11SP3 = #{@upgradeModeOES11SP3},
        upgradeModeOES2018 = #{@upgradeModeOES2018},
        upgradeModeOES2018SP1 = #{@upgradeModeOES2018SP1},
        upgradeModeOES2018SP2 = #{@upgradeModeOES2018SP2}"
      )
      Builtins.y2milestone("sms:oldSettings: %1", @oldSettings)

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      # determine if sms configuration already exists
      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.sms.SERVICE_CONFIGURED"))
      )
      service_configured = Builtins.tolower(service_configured)

      if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 || @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 || @upgradeModeOES2018SP2
        Ops.set(
          @Settings,
          "CONFIG_SMS_LDAP_SERVER",
          Ops.get_string(@oldSettings, "CONFIG_SMS_LDAP_SERVER", "")
        )
      else
        if service_configured != "yes"
          server_list = NovellLDAP.GetLocalServers(nil)
          if Ops.greater_than(Builtins.size(server_list), 0)
            Ops.set(
              @Settings,
              "CONFIG_SMS_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          else
            server_list = NovellLDAP.GetRemoteServers(nil)
            Ops.set(
              @Settings,
              "CONFIG_SMS_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          end
        else
          if Mode.normal
            name = "SMS"
            if !Popup.YesNo(
                Builtins.sformat(
                  _(
                    "It appears that %1 has already been configured.  To reconfigure %2 please exit and use iManager. \nIf %3 has been uninstalled and is being reinstalled you may continue.  Continue configuration?"
                  ),
                  name,
                  name,
                  name
                )
              )
              return false
            end
          end
          Ops.set(
            @Settings,
            "CONFIG_SMS_LDAP_SERVER",
            Convert.to_string(
              SCR.Read(path(".sysconfig.sms.CONFIG_SMS_LDAP_SERVER"))
            )
          )
        end
      end
      Builtins.y2debug("SMS READ Complete:%1", @Settings)

      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      #    if (Mode::normal())
      #    {
      #        if (! NovellPatterns::EnsurePatternInstalled("novell-sms") )
      #        {
      #           return false;
      #           return true;   // for debug only
      #        }
      #    }

      @globals_configured = true

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end



    # Checks a string for output indicating insufficient rights.
    #
    # @param [String] msg error message from stdout/stderr
    # @return [Boolean] true if the error message is due to insufficient rights
    def insufficient_rights_error(msg)
      return false if msg == nil || msg == ""

      idx = Builtins.search(msg, "Insufficient access")
      return true if idx != nil && Ops.greater_or_equal(idx, 0)

      idx = Builtins.search(msg, "no access (-672)")
      return true if idx != nil && Ops.greater_or_equal(idx, 0)

      false
    end


    # Write all sms settings
    # @return true on success
    def Write
      Builtins.y2milestone("Sms::Write started")
      pattern = "novell-sms"

      Wizard.DisableAbortButton

      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end

      sl = 500
      cmd = ""

      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone("Sms:setting ABORT_OES_INSTALL is set to 'yes'.")
        return true
      end
      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "sms:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end

      Builtins.y2milestone("sms:xadMode:%1", @xadMode)

      if  !@upgradeModeOES11SP2 &&
          !@upgradeModeOES2015 &&
          !@upgradeModeOES2015SP1 &&
          !@upgradeModeOES11SP3 && 
          !@upgradeModeOES2018 &&
          !@upgradeModeOES2018SP1 &&
          !@upgradeModeOES2018SP2 

	# skip config on upgrade for the last two oes versions
        if !Ops.get(NovellAddonInstall.products_to_configure, "sms", true)
          Builtins.y2milestone("SMS -- configure later")
          return true
        end

        # If Aquila did not install properly, then we don't want to proceed at this point.
        Builtins.y2milestone(
          "sms:testing to see if eDir configured properly. Xad mode: %1",
          @xadMode
        )
        if @xadMode == true
          Builtins.y2milestone(
            "sms:eDir service_configured (mem):%1",
            Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "")
          )
          Builtins.y2milestone(
            "sms:eDir service_configured (file):%1",
            Convert.to_string(
              SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
            )
          )
          if Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "") != "yes" &&
              Convert.to_string(
                SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
              ) != "yes"
            if EdirUtils.EdirConfigProblemPrompt("SMS")
              Builtins.y2milestone(
                "sms:aborting SMS write by user choice since eDirectory was not configured."
              )
              return true
            end
          end
        end

        # Sms read dialog caption
        caption = _("Saving SMS Configuration")

        # TODO FIXME And set the right number of stages
        steps = 1

        Builtins.sleep(sl)

        # TODO FIXME Names of real stages
        # We do not set help text here, because it was set outside
        Progress.New(
          caption,
          " ",
          steps,
          [
            # Progress stage 1/1
            _("Write the settings")
          ],
          [
            # Progress step 1/1
            _("Writing the settings..."),
            # Progress finished
            _("Finished")
          ],
          ""
        )

        # write the settings
        return false if Abort()
        Progress.NextStage

        #apply settings
        SCR.Write(path(".sysconfig.sms.SERVICE_CONFIGURED"), "no")

        # determine IP addresses
        server_list = Ops.get_list(
          NovellLDAP.Settings,
          "CONFIG_LDAP_SERVERS_LIST",
          []
        )
        primary_server = NovellLDAP.GetAddressServer(
          Ops.get_string(@Settings, "CONFIG_SMS_LDAP_SERVER", ""),
          server_list
        )
        fallback_list = NovellLDAP.GetFallbackServerList(
          Ops.get_string(@Settings, "CONFIG_SMS_LDAP_SERVER", ""),
          server_list
        )
        Builtins.y2milestone("Primary server: %1", primary_server)
        Builtins.y2milestone("Fallback server_list: %1", fallback_list)

        full_admin_context = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        admin_pwd = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )

        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "Sms:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            full_admin_context = NovellUtils.Nds2Ldap(xad_tree_admin_context)
            admin_pwd = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
          end
        end
        Builtins.y2milestone("Sms:admin:%1", full_admin_context)

        schema_file = "/opt/novell/sms/schema/sms.ldif"

        cmd = Builtins.sformat(
          "/opt/novell/eDirectory/bin/ice -S LDIF -f %1 -D LDAP -s %2 -p %3 -d '%4' -L",
          schema_file,
          Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
          Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636),
          full_admin_context
        )
        Builtins.y2milestone("Executing %1", cmd)

        upgradeStr = ""
        if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1  || @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 || @upgradeModeOES2018SP2
          upgradeStr = "UPGRADE='1' "
        end

        insufficient_rights = false
        if @xadTestMode
          Builtins.y2milestone(
            "xad:Sms:executing %1",
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(upgradeStr, "ADM_DEST_PASSWD='"), admin_pwd),
                "' "
              ),
              cmd
            )
          )
        end
        mval = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(upgradeStr, "ADM_DEST_PASSWD='"), admin_pwd),
                "' "
              ),
              cmd
            )
          )
        )
        if Ops.get_integer(mval, "exit", -1) != 0
          if insufficient_rights_error(Ops.get_string(mval, "stderr", "")) == true
            insufficient_rights = true
          end

          Builtins.y2milestone(
            "novell-sms: Failed (error=%1) to extend schema using default IP address / hostname: %2, using file %3",
            Ops.get_integer(mval, "exit", -1),
            Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
            schema_file
          )

          Builtins.foreach(fallback_list) do |server_info|
            Builtins.y2milestone("novell-sms: Retrying with fallback address.")
            cmd = Builtins.sformat(

 	      "/opt/novell/eDirectory/bin/ice -S LDIF -f %1 -D LDAP -s %2 -p %3 -d '%4' -L",
              schema_file,
              Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
              Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
              full_admin_context
            )
            Builtins.y2milestone("novell-sms:Executing %1", cmd)
            if @xadTestMode
              Builtins.y2milestone(
                "xad:Sms:executing %1",
                Ops.add(
                  Ops.add(
                    Ops.add(Ops.add(upgradeStr, "ADM_DEST_PASSWD='"), admin_pwd),
                    "' "
                  ),
                  cmd
                )
              )
            end
            mval2 = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(
                  Ops.add(
                    Ops.add(Ops.add(upgradeStr, "ADMIN_DEST_PASSWD='"), admin_pwd),
                    "' "
                  ),
                  cmd
                )
              )
            )
            if Ops.get_integer(mval2, "exit", -1) != 0
              if insufficient_rights_error(Ops.get_string(mval2, "stderr", "")) == true
                insufficient_rights = true
              end
              Builtins.y2milestone(
                "novell-sms:Failed (error=%1) to extend schema using fallback IP address / hostname: %2, using file %3",
                Ops.get_integer(mval2, "exit", -1),
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                schema_file
              )
            else
              raise Break
            end
          end
        end
        if Ops.get_integer(mval, "exit", -1) != 0
          if (Mode.normal || Mode.installation && Stage.normal) &&
              insufficient_rights == true
            warn = Ops.add(
              Builtins.sformat(
                _("The user %1 has insufficient rights to extend schema.\n"),
                full_admin_context
              ),
              _(
                "This may not be a problem if joining a tree where the schema is already extended."
              )
            )

            Builtins.y2milestone("novell-sms.write:%1", warn)
          end
          Builtins.y2milestone(
            "novell-sms.write:Failed to extend schema for Novell SMS. Schema may already be extended if joining a NetWare Tree."
          )
        end
      else
        Builtins.y2milestone("novell-sms.write:skipping config for upgrade.")
      end

      # Write the /etc/sysconfig file
      SCR.Write(
        path(".sysconfig.sms.CONFIG_SMS_LDAP_SERVER"),
        Ops.get_string(@Settings, "CONFIG_SMS_LDAP_SERVER", "")
      )
      Builtins.y2debug("SMS(Write): %1", @Settings)

      Builtins.sleep(sl)

      service="novell-smdrd"
      if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 || @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 || @upgradeModeOES2018SP2
        NovellUtils.SystemdStartorRestartService(service,"yes")
      else
        NovellUtils.SystemdStartorRestartService(service,"no")
      end

      if  !@upgradeModeOES11SP2 ||
          !@upgradeModeOES2015 ||
          !@upgradeModeOES2015SP1 ||
          !@upgradeModeOES11SP3 || 
          !@upgradeModeOES2018 ||
          !@upgradeModeOES2018SP1 ||
          !@upgradeModeOES2018SP2 
        # skip config on upgrade of last two versions
        cmd = "/opt/novell/sms/bin/smsconfig -l tsafs"
        Builtins.y2milestone("Executing: %1", cmd)
        rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      end

      # open port 40193 for smdr daemon
      NovellUtils.SetFirewallPort(40193, :tcp, true)

      # success
      SCR.Write(path(".sysconfig.sms.SERVICE_CONFIGURED"), "yes")

      # Flush the sysconfig file.
      SCR.Write(path(".sysconfig.sms"), nil)

      return false if Abort()
      # Progress finished
      Progress.NextStage
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()
      true
    end


    # Get all sms settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "sms", false)
        return false
      end

      Ops.set(
        @Settings,
        "CONFIG_SMS_LDAP_SERVER",
        Ops.get_string(settings, "ldap_server", "")
      )

      @globals_configured = true

      true
    end

    # Dump the sms settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-sms") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "sms", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "sms", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_SMS_LDAP_SERVER", "")
        )
      end

      deep_copy(settings)
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)
        # if( OesLdap::autoyast_tool == true && OesLdap::local_server == true )
        #             summary = Summary::AddListItem(summary, sformat(_("LDAP IP address: %1"),
        # 	            OesLdap::LOCAL_HOST) );
        #         else
        #             summary = Summary::AddListItem(summary, sformat(_("LDAP IP address: %1"),
        # 	            Settings["CONFIG_LDAP_SERVER_IP"]:"") );


        # Configuration summary text for autoyast
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("LDAP Server: %1"),
            Ops.get_string(@Settings, "CONFIG_SMS_LDAP_SERVER", "")
          )
        )


        summary = Summary.CloseList(summary)
      end

      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :upgradeModeOES11SP2, :type => "boolean"
    publish :variable => :upgradeModeOES11SP3, :type => "boolean"
    publish :variable => :upgradeModeOES2015, :type => "boolean"
    publish :variable => :upgradeModeOES2015SP1, :type => "boolean"
    publish :variable => :upgradeModeOES2018, :type => "boolean"
    publish :variable => :upgradeModeOES2018SP1, :type => "boolean"
    publish :variable => :upgradeModeOES2018SP2, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  Sms = SmsClass.new
  Sms.main
end
